/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/vfs.h>
#include <sys/wait.h>

#include "main.h"
#include "config.h"
#include "fs.h"
#include "misc.h"
#include "strings.h"

typedef unsigned int xid_t;
typedef unsigned int nid_t;

#include <vserver.h>

char *get_xid(char *name)
{
	char *output = NULL;
	char *xidfile;

	xidfile = concat(global_settings.config_dir, "/", name, "/context", NULL);

	if(file_exist(xidfile))
	{
		output = file_read(xidfile);
	}
	else
	{
		output = strdup("N/A\n");
	}

	free(xidfile);
	return output;
}


char *get_rlimit(char *name, char *limit)
{
	char *output = NULL;
	char *flagfile;
	
	flagfile = concat(global_settings.config_dir, "/", name, "/rlimits/", limit, NULL);

	if(file_exist(flagfile))
	{
		output = file_read(flagfile);
	}
	else
	{
		output = strdup("N/A\n");
	}

	free(flagfile);
	return output;
}

char *get_state(char *name)
{
	char *output;
	char *runfile;
	
	runfile = concat(global_settings.run_dir, "/", name, NULL);
	
	if(file_exist(runfile))
	{
		output = strdup("Online\n");
	}
	else
	{
		output = strdup("Offline\n");
	}

	free(runfile);

	return output;
}

char *get_infos(char *name)
{
	char *output = NULL;
	char *path, *context;
	
	path = concat(global_settings.config_dir, "/", name, "/context", NULL);
	context = file_read(path);
	trim_string(context);
	free(path);

	if(context != NULL)
	{
		path = concat("/proc/virtual/", context, "/limit", NULL);	
		output = file_read(path);
		free(path);
		free(context);
	}
	
	if(output == NULL)
	{
		output = strdup("N/A\n");
	}


	return output;
}

char *get_ips(char *name)
{
	struct dirent *dir_info;
	char path[BUF_SIZE+1];
	char *ip, *device, *alias, *netmask, *bcast, *output = NULL;
	int output_len = 0;
	DIR *dirst;
	
	
	sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces", NULL);
	dirst = opendir(path);


	if(dirst != NULL)
	{

		while ((dir_info = readdir(dirst)) != NULL)
		{
		
			if(dir_info->d_name[0] != '.')
			{

				sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, NULL);
	
				if(dir_exist(path))
				{
					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, "/ip", NULL);
					ip = file_read(path);

					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, "/dev", NULL);
					device = file_read(path);

					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, "/name", NULL);
					alias = file_read(path);

					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, "/mask", NULL);
					netmask = file_read(path);

					sconcat(path, BUF_SIZE, global_settings.config_dir, "/", name, "/interfaces/", dir_info->d_name, "/bcast", NULL);
					bcast = file_read(path);
					
					if(device == NULL)
						device = strdup("nodev");
					if(alias == NULL)
						alias = strdup("noname");
					if(netmask == NULL)
						netmask = strdup("nomask");
					if(bcast == NULL)
						bcast = strdup("nobcast");


	
					if(ip != NULL)
					{
	
						trim_string(ip);
						trim_string(device);
						trim_string(alias);
						trim_string(netmask);
						trim_string(bcast);

						output = (char *)realloc(output, output_len + strlen(ip) + 1 
											+ strlen(device) + 1 
											+ strlen(alias)  + 1
											+ strlen(netmask) + 1 
											+ strlen(bcast) + 3);

						strcpy(output+output_len, ip);
						output_len += strlen(ip);

						output[output_len] = ' ';
						output_len++;

						strcpy(output+output_len, device);
						output_len += strlen(device);

						output[output_len] = ' ';
						output_len++;

						strcpy(output+output_len, alias);
						output_len += strlen(alias);

						output[output_len] = ' ';
						output_len++;

						strcpy(output+output_len, netmask);
						output_len += strlen(netmask);

						output[output_len] = ' ';
						output_len++;

						strcpy(output+output_len, bcast);
						output_len += strlen(bcast);

						output[output_len] = ' ';
						output_len++;

						output[output_len] = '\n';
						output[output_len+1] = 0;
	

	
						free(ip);
					}



					free(device);
					free(alias);
					free(netmask);				
					free(bcast);				

				}

			}


		}

		closedir(dirst);
	
	}


	if(output == NULL)
	{
		output = strdup("N/A\n");
	}


	return output;
}

char *get_flags(char *name)
{
	char *output;
	char *flagfile;
	
	flagfile = concat(global_settings.config_dir, "/", name, "/flags", NULL);

	if(file_exist(flagfile))
	{
		output = file_read(flagfile);
	}
	else
	{
		output = strdup("N/A\n");
	}

	free(flagfile);
	return output;
}

char *get_schedule(char *name)
{
	char *output, *interval;
	char *schedfile;
	int fd;

	schedfile = concat(global_settings.config_dir, "/", name, "/schedule", NULL);

	fd = open(schedfile, O_RDONLY);

	if(fd != -1)
	{

		output = readline(fd);

		if(output != NULL)
		{
			output = string_append(output, "-");
			interval = readline(fd);

			if(interval != NULL)
			{
				output = string_append(output, interval);
				free(interval);		
			}
		
			output = string_append(output, "\n");
		}
		
		close(fd);
	}
	else
	{
		output = strdup("N/A\n");
	}

	free(schedfile);

	return output;
}

char *get_hostname(char *name)
{
 	char *output;
 	char *hostfile;
 	  
   	hostfile = concat(global_settings.config_dir, "/", name, "/uts/nodename", NULL);

	if(file_exist(hostfile))
	{
		output = file_read(hostfile);
	}
	else
	{
		output = strdup("Not Set\n");
	}

	free(hostfile);
 	return output;
}

char *get_space(char *name)
{

	char *path, *dlimits_file, *output = NULL;
	int space_total = 0, space_used = 0, space_free = 0;
      struct vc_ctx_dlimit limits;
	xid_t xid;

	xid = vserver_xid(name);

	path = concat(global_settings.root_dir, "/", name, NULL);

	if(vc_get_dlimit(path, xid, 1, &limits) != -1)
	{

		space_total = limits.space_total/1024;
		space_used = limits.space_used/1024;
		space_free = space_total-space_used;
	}
	else
	{
		free(path);
		path = concat(global_settings.config_dir, "/", name, "/", "dlimits/0/space_total", NULL);
		dlimits_file = file_read(path);

		if(dlimits_file != NULL)
		{
			trim_string(dlimits_file);
			space_total = atoi(dlimits_file)/1024;
			free(dlimits_file);
		}
		else
		{
			space_total = -1;
		}
		
		space_used = -1;
		space_free = -1;
	}
		
	free(path);



	output = malloc(BUF_SIZE+1);
	snprintf(output, BUF_SIZE, "%d %d %d\n", space_total, space_used, space_free);



	return output;
}



