/*  OpenVCP
 *  Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>


#include "main.h"
#include "fs.h"
#include "strings.h"
#include "misc.h"
#include "config.h"
#include "request.h"


struct config_st global_settings;


#ifdef DEBUG

void gnutls_log(int level, const char *msg)
{

	 syslog(LOG_INFO, "[GNUTLS] [%d] [%s]\n", level, msg);

}
#endif


void sigterm_handler(int signal)
{

	exit(0);
}


int main(int argc, char **argv)
{

        pid_t daemon_pid = 0;
        char *cmd, *pid_string, **arguments;
        unsigned int ret, size;
        unsigned int args, option;
        int verbose = 0;

        int server_socket, client_socket;
        struct sockaddr_in server_addr, client_addr;

        void *session;

        #ifdef HAVE_LIBGNUTLS

                gnutls_anon_server_credentials_t anoncred;
                gnutls_dh_params_t dh_params;

        #endif

        memset(&server_addr, 0, sizeof(server_addr));
        memset(&client_addr, 0, sizeof(server_addr));

        while ((option = getopt(argc, argv, "v")) != -1)
        {
                switch(option)
                {
                        case 'v': 
                                verbose = 1;
                                break;
                        case 'h':
                                usage(argv[0]);
                                exit(-1);
                                break;
				default:
                                usage(argv[0]);
                                exit(-1);	
                }

        }
        
        if(verbose == 1)
                openlog("openvcpd", LOG_PID | LOG_PERROR, LOG_DAEMON);
        else
                openlog("openvcpd", LOG_PID, LOG_DAEMON);                

 
        syslog(LOG_INFO, "OpenVCP Daemon %s started", PACKAGE_VERSION);

        parse_config(OPENVCP_CONFIG, &global_settings);



        signal(SIGCHLD, SIG_IGN);
        signal(SIGTERM, sigterm_handler);

        if(verbose != 1)
        {
                if((daemon_pid = fork()) == -1)
                {
                        syslog(LOG_ERR, "Could not fork()");
                        exit(-1);
                }
        }


        if(daemon_pid == 0)
        {


                #ifdef HAVE_LIBGNUTLS

                        gnutls_global_init ();

                        #ifdef DEBUG
                        	       gnutls_global_set_log_level(10);
                                       gnutls_global_set_log_function(&gnutls_log);
                        #endif
        
                        gnutls_anon_allocate_server_credentials (&anoncred);

                        gnutls_dh_params_init (&dh_params);
                        gnutls_dh_params_generate2 (dh_params, 1024);

                        gnutls_anon_set_server_dh_params (anoncred, dh_params);

                #endif

                server_socket = socket(PF_INET, SOCK_STREAM, 0);
                server_addr.sin_addr.s_addr = inet_addr(global_settings.ip);
                server_addr.sin_port = htons(atoi(global_settings.port));


                fcntl(server_socket, F_SETFD, 1);

                option = 1;
                setsockopt(server_socket, SOL_SOCKET, SO_REUSEADDR, &option, sizeof(option));


                ret = bind(server_socket, (struct sockaddr *)&server_addr, sizeof(server_addr));

                if(ret == -1)
                {
                        syslog(LOG_ERR, "Port already in use");
                        close(server_socket);
                        exit(-1);
                }

                signal(SIGCHLD, SIG_IGN);
                on_exit(cleanup_socket, &server_socket);


                listen(server_socket, 5);

                size = sizeof(client_addr);


                while(1)
                {

                        client_socket = accept(server_socket, (struct sockaddr *)&client_addr, &size);
                        fcntl(client_socket, F_SETFD, 1);

                        syslog(LOG_INFO, "Connection from: %s", inet_ntoa(client_addr.sin_addr));


                        #ifdef HAVE_LIBGNUTLS
                                session = NULL;
                        #endif
                        

                        cmd = ovcp_readline(client_socket, session);


                        if(cmd != NULL)
                        {
                                arguments = split_string(cmd, ' ', &args);
                                free(cmd);

                                if(arguments != NULL)
                                {



                                         if(args >= 1 && strcmp(arguments[0], "TLS") == 0)
                                         {

                                                ovcp_write(client_socket, "OK\n", 4, session);                                            

                                                #ifdef HAVE_LIBGNUTLS
                                                      if(session == NULL)
                                                      {

                                                           session = (void *)tls_session_new (client_socket, anoncred);
                                                           ret = gnutls_handshake((*(gnutls_session_t *)session));

							   syslog(LOG_INFO, "Handshake complete (%d)", ret);
						      }
                                                #else
                                                      ovcp_write(client_socket, "Not Supported\n", 15, session);
                                                #endif 

                                        }
                                        if(args >= 1 && strcmp(arguments[0], "PLAINTEXT") == 0)
                                        {
        
                                                      ovcp_write(client_socket, "OK\n", 4, session);                                            
                                        }



                                        free_strings(arguments, args);

                                 }
                        }



                        cmd = ovcp_readline(client_socket, session);


                        if(cmd != NULL)
                        {
                                arguments = split_string(cmd, ' ', &args);
                                free(cmd);

                                if(arguments != NULL)
                                {

                                        if(args >= 2 && strcmp(arguments[0], "PASSWORD") == 0)
                                        {
                                                if(strcmp(arguments[1], global_settings.password) == 0)
                                                {
                                                        ovcp_write(client_socket, "OK\n", 3, session);

                                                        if(fork() == 0)
                                                        {
                                                                handle_request(client_socket, session);
                                                        }

                                                }
                                                else
                                                {
                                                        sleep(3);
                                                        ovcp_write(client_socket, "Failed\n", 7, session);

                                                }

                                         }
                                         else
                                         {
                                                ovcp_write(client_socket, "Command not understood\n", 23, session);
                                         }


                                        free_strings(arguments, args);
                                }

                        }



                        #ifdef HAVE_LIBGNUTLS

                        if(session != NULL)
                        {
                                gnutls_deinit (*((gnutls_session_t *)session));
                                free(session);
                        }

                        #endif

                        close(client_socket);
                        

                }

                close(server_socket);

                #ifdef HAVE_LIBGNUTLS
                        gnutls_anon_free_server_credentials (anoncred);
                        gnutls_global_deinit ();
                #endif

        }
	else
	{
		pid_string = dec2str(daemon_pid);
		string_append(pid_string, "\n");	

		file_write(global_settings.pid_file, pid_string);

		free(pid_string);
	}


        return 0;
}




int handle_request(int client_socket, void *session)
{


        char *request;
        char *response;
        unsigned int args;
        int res;
        char **arguments;

        fd_set client_fdset;
        struct timeval timeout;

        on_exit(cleanup_socket, &client_socket);

        while(1)
        {
                FD_ZERO(&client_fdset);
                FD_SET(client_socket, &client_fdset);

                timeout.tv_sec = 4;
                timeout.tv_usec = 50000000;

                res = select(client_socket+1, &client_fdset, NULL, NULL, &timeout);

                response = NULL;


                if(res != 0 && res != -1)
                {

                        request = ovcp_readline(client_socket, session);



                        if(request != NULL)
                        {
                                arguments = split_string(request, ' ',&args);

                                trim_string(request);
                                syslog(LOG_DEBUG, "Request: \"%s\"\n", request);

                                free(request);

                                if(strcmp(arguments[0], "CONTROL") == 0)
                                {
                                        response = vserver_control(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "NODE") == 0)
                                {
                                        response = node_get(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "LIST") == 0)
                                {
                                        response = vserver_list(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "SETUP") == 0)
                                {
                                        response = vserver_setup(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "SET") == 0)
                                {
                                        response = vserver_set(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "GET") == 0)
                                {
                                        response = vserver_get(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "TRAFFIC") == 0)
                                {
                                        response = vserver_traffic(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "REMOVE") == 0)
                                {
                                        response = vserver_remove(arguments, args);
                                }
                                else
                                if(strcmp(arguments[0], "EXIT") == 0)
                                {
                                        ovcp_close(client_socket, session);
                                        exit(1);
                                }
                                else
                                if(strcmp(arguments[0], "VERSION") == 0)
                                {
                                        response = strdup(PACKAGE_VERSION);
                                }
                                else
                                {
                                        response = strdup("Command not understood\n");
                                }

                                free_strings(arguments, args);


                                if(response != NULL)
                                {

                                        FD_ZERO(&client_fdset);
                                        FD_SET(client_socket, &client_fdset);

                                        timeout.tv_sec = 4;
                                        timeout.tv_usec = 5000000;

                                        res = select(client_socket+1, NULL, &client_fdset, NULL, &timeout);

                                        if(res != 0 && res != -1)
                                        {
                                                ovcp_write(client_socket, response, strlen(response), session);

                                                trim_string(response);
                                                syslog(LOG_DEBUG, "Response: \"%s\"\n", response); 
                                       }


                                        free(response);
                                }
                                else
                                {
                                        ovcp_write(client_socket, "INTERNAL ERROR\n", 15, session);
                                        syslog(LOG_DEBUG, "Response: \"INTERNAL ERROR\"\n"); 
                                }

                        }
                        else
                        {
                                ovcp_write(client_socket, "Disconnected\n", 13, session);
                                ovcp_close(client_socket, session);
                                exit(0);
                        }
                }
                else
                {

                        ovcp_close(client_socket, session);
                        exit(1);
                }

        }


}

#ifdef HAVE_LIBGNUTLS

gnutls_session_t *tls_session_new (int client_socket, gnutls_anon_server_credentials_t anoncred)
{
        gnutls_session_t *session;
        const int kx_prio[] = { GNUTLS_KX_ANON_DH, 0 };

        session = (gnutls_session_t *)malloc(sizeof(gnutls_session_t));

        gnutls_init (session, GNUTLS_SERVER);

        gnutls_set_default_priority (*session);
        gnutls_kx_set_priority (*session, kx_prio);

        gnutls_credentials_set (*session, GNUTLS_CRD_ANON, anoncred);

        gnutls_dh_set_prime_bits (*session, 1024);

        gnutls_transport_set_ptr (*session, (gnutls_transport_ptr_t) client_socket);

        return session;
}

#endif

void usage(char *name)
{
        printf("Usage: %s [OPTION]\n\n-v\t\tbe verbose, log to stderr\n-h\t\tdisplay this help\n\n", name);
}

void cleanup_socket(int args, void *socket_ptr)
{
        int *socket = (int *)socket_ptr;

        close(*socket);

}
